"""
Support script for building autograders.
"""
import click
from zipfile import ZipFile
import os
from typing import List, Union, Tuple


def get_all_file_paths(directory : str) -> List[str]:
    """
        Grabs all the files in the directory provided, to write them out to the tests directory.
        While it will get files in sub directories, it does not currently handle
        keeping those subdirectories when writing!
    Args:
        directory (str): directory to scan

    Returns:
        List[str]: a list of files found
    """
    # initializing empty file paths list
    file_paths = []

    # crawling through directory and subdirectories
    for root, directories, files in os.walk(directory):
        for filename in files:
            if "__" in root: continue # skip special python directories TODO: improve this!
            # join the two strings in order to form the full filepath.
            filepath = os.path.join(root, filename)
            file_paths.append(filepath)

    # returning all file paths
    return file_paths

def new_test_loc(file: str) -> str:
    """converts a file found to be put into the tests directory
    TODO: this function should be updated to be more flexible.
    Args:
        file (str): the file to convert path to tests directory in the zip

    Returns:
        str: tests/file
    """
    return "tests/" + os.path.basename(file)


def check_or_get(file : str, override:List[str]) -> Tuple[str, str]:
    """Checks to see if the file is in the override list, else
    provides a default file from the templates directory
    Args:
        file (str): the file to process
        override (List[str]): the override list of files

    Returns:
        Tuple[str, str]: name of the file to write, and the path to the file
    """
    for f in override:
        if file in f:
            return (f, f)
    # not found use common list
    dirname = os.path.dirname(__file__)
    return (file, os.path.join(dirname, f"templates/{file}"))

@click.command()
@click.option("--out", "-o", default="autograder.zip")
@click.argument("dir")
@click.argument("override", nargs=-1)
def build_autograder(out: str, dir: str, override: Union[List[str], None]):
    """Commandline tool for generating autograder_files. For the most part
    the default execution will work. Reads files in dir, addes them 
    to a zipfile (out), and can add some of the standard four autograder files
    incase they want to be replaced
    Args:
        out (str): file to save
        dir (str): tests directory
        override (Union[List[str], None]): List of files to replace the default autograder files, names must match
    """
    paths = get_all_file_paths(dir)
    common_files_default = ("requirements.txt", "run_autograder", "run_tests.py", "setup.sh")


    with ZipFile(out, 'w') as zip:
        for file in paths:
            zip.write(file, arcname=new_test_loc(file))
        for file in common_files_default:
            name, full = check_or_get(file, override)
            zip.write(full, arcname=name)


if __name__ == "__main__":
    build_autograder()








from typing import Dict, List
from gradescope_utils.autograder_utils.json_test_runner import JSONTestRunner

## common TIER 

COMMON_ONE = "learning"
COMMON_TWO = "approaching"
COMMON_THREE = "meets"
COMMON_FOUR = "exceeds"

class tier(object):
    """An wrapper that modifies the tags attribute to include a dictionary specifying the tier
    """
    def __init__(self, val : str):
        self.tier = val


    def __call__(self, func):
        if hasattr(func, "__tags__") :
            func.__tags__ = {"tier": self.tier, "tags": func.__tags__}
        else:
            func.__tags__ = {"tier" : self.tier}
        return func



class TierMasteryJSONTestRunner(JSONTestRunner):
    failed = None

    def __init__(self, tiers=(COMMON_ONE, COMMON_TWO, COMMON_THREE, COMMON_FOUR), score_per_tier = 1, stop_grading=True, **kargs):
        self.tiers = tiers
        self.stop_grading = stop_grading
        self.score_per_tier = score_per_tier
        if 'post_processor' in kargs:
            self.second_processor = kargs['post_processor']
            del kargs['post_processor']
        else:
            self.second_processor = lambda a : a ## a do nothing function
        super().__init__(post_processor=self.post_processor, **kargs)


    def post_processor(self, json: Dict) -> Dict:
        """A post-processing wrapper for gradescope_utils.autograder_utils.json_test_runner.JSONTestRunner. The wrapper
        will either use the default config file, or a config file set before the post processor call 

        Args:
            json (Dict): json graded test result generated by JSONTestRunner

        Returns:
            Dict: A new modified dict for printing out
        """
        json["score"] = sum((self.__check_tests(self._filter(json['tests'], tier), tier)   for tier in self.tiers))
        return self.second_processor(json)

    def _filter(self, tests: List, tier: str) -> List:
        """Helper method to filter a group of tests based on the tier (due to complexity of the if statement)

        Args:
            json (Dict): the results dictionary being processed

        Returns:
            List: the tests that match the filter based on the tier 
        """
        return [x for x in tests if ("tags" in x) and ("tier" in x["tags"]) and (x['tags']['tier'] == tier)]


    def __check_tests(self, tests: List, tier: str) -> int:
        """Loops through all the tests for the tier. If the previous tier has
        already failed, then cleans up the info for that tier

        Args:
            tests (List): a list of tests to check
            tier (str): the tier mainly used for setting the self.failed and the common update on test names

        Returns:
            int: the score 1 if all tests passed, or 0 if *any* test failed
        """
        if len(tests) < 1 : return 0 # don't score empty tiers
        score = self.score_per_tier
        if self.failed is not None and self.stop_grading: 
            self.__remove_tier(tests)
            score = 0
        fail_count = sum((1 for test in tests if test["status"] == "failed"))         
        if fail_count > 0:
            self.failed = tier
            score = 0
        self.__common_update(tests, tier)
        return score 

    def __common_update(self, tests: List, tier: str) -> None:
        """Updating the autograder tags to better match Mastery Tier based grading. 
        Assumed a tier for them to be grouped by

        Args:
            tests (List): The tests to update
            tier (str): the tier they are all associated with
        """
        for test in tests:
            if "score" in test:
                del test["score"] #not needed as there will be one at the top level
            test["name"] = f"{tier.capitalize()}: {test['name']}"
            
            if "tags" in test["tags"]:
                test["tags"] = tuple(test["tags"]["tags"]) 
            else:
                # remove the tag used for the tier
                del test["tags"]


    def __remove_tier(self, tests: List) -> None:
        """
        Removes the test results from that tier, even they have been completed. Often
        called after one tier has failed already.

        Args:
            tests (List): A group of tests to remove results from, and mark all as failed.
        """
        for test in tests:
           test["status"] = "failed"
           test["output"] = f"Test Skipped: {str(self.failed).capitalize()} Tier needs corrected and resubmitted first"


"""
A simple tool set developed for non-traditional grading and other grading 
tools within the grade-scope autograder environment.

Features:
    Tier Mastery Grading: 
    Focuses on mastery based grading in grading tiers, meaning
    grades are corrected, so only the tests are passed in order based on groups. If any
    test in a group stops, so does all grading until the previous group is fixed. 

    Common tiers are (COMMON_ONE, COMMON_TWO, COMMON_THREE, COMMON_FOUR) which is
    "Learning", "Approaching", "Meets", "Exceeds". To tag a test in a tier use the
    tier decorator. An example of a common test setup with tier grading would be as follows

    Example:
    from gradescope_utils.autograder_utils.decorators import number, tags
    from neugs_utils import tier, COMMON_ONE, COMMON_TWO, COMMON_THREE

    class TestOne(unittest.TestCase):
        @tier(COMMON_THREE)
        @tags("Learning")  #tags should come *after* tiers if they are used at all
        @number(3.0)
        def test_random(self):
            result = 6
            for i in (6, 5, 61):
                self.assertEqual(result, i, "this is my message, that display due to TWO failing")

        @tier(COMMON_TWO)
        @number(2.0)
        def test_approaching(self):
            result = 3
            self.assertEqual(result, 5, "this should fail")

        @tier(COMMON_TWO)
        @number(2.0)
        def test_some_other_approaching(self):
            result = 5
            self.assertEqual(result, 5, "this should pass")

        @tier(COMMON_ONE)
        @number(1.0)
        def test_valid(self):
            self.assertEqual(5, 5, "really!")

        @tier(COMMON_ONE)
        @number(1.1)
        def test_valid2(self):
            self.assertEqual(6, 6, "really!")

    Given the example above, and assuming default points of 1 point per tier, the above student would earn
    1 point, and will be encouraged to submit again to completed Tier Two and Tier 3

    In run_test.py make sure to change JSONTestRunner to TierMasteryJSONTestRunner
        
    suite = unittest.defaultTestLoader.discover('tests')
    with open('/autograder/results/results.json', 'w') as f:
        TierMasteryJSONTestRunner(visibility='visible', stream=f).run(suite)

Utilities:
    There are also a number of additional utility functions added in common_tests.py and context_managers.py.
    These are meant to be for utility to help with common grading tasks. 


This module is still in early stages of development! 

Todo:
    * Add standards based grading / tagging

"""


from typing import Union, List, Tuple, Set, Any
from .tier_grading import TierMasteryJSONTestRunner, tier, \
    COMMON_ONE, COMMON_TWO, COMMON_THREE, COMMON_FOUR


__version__ = "0.0.6"


def common_msg(msg : str, expected : Any, actual : Any) -> str:
    """Generates a common message format of 
       msg
         expected: val
         actual:  val

    Args:
        msg (str): An informative message for the student
        expected (Any): the expected value
        actual (Any): the actual value

    Returns:
        str: a string formatted for student view
    """
    return f"{msg}\n\tExpected: {expected}\n\t  Actual: {actual}"


def strip_prompt(prompt: str, outputs: Tuple[str]) -> List[str]:
    """Removes prompts from every output line provided

    Args:
        prompt (str): the prompt to remove (needs to be exact)
        outputs (Tuple[str]): the output line

    Returns:
        List[str]: a list of outputs with the prompts removed
    """
    return [output.replace(prompt, '').strip() if prompt in output else output for output in outputs]


def strip_prompts(prompts: Set[str], outputs: Tuple[str]) -> List[str]:
    """Removes a set of prompts from every output line as found

    Args:
        prompts (Set[str]): a list of prompts to remove
        outputs (Tuple[str]): a list of output lines

    Returns:
        List[str]: a set of output lines with prompts removed
    """
    out = list(outputs)
    for prompt in prompts:
        out = strip_prompt(prompt, tuple(out))
    return out



import os
import sys
#up1 = os.path.abspath('../..')
#print("Add path for test purposes " + up1)
#sys.path.insert(0, up1)




import unittest
from gradescope_utils.autograder_utils.decorators import number, tags
from neugs_utils.context_managers import Capturing
from neugs_utils import tier, COMMON_ONE, COMMON_TWO, COMMON_THREE
from random import randint

def some_func():
    val = input("this is my input")
    print(f"this is your input: {val}\n{val}" )

class TestOne(unittest.TestCase):

    @tier(COMMON_ONE)
    def test_capture(self):
        with Capturing("hello") as output:
            some_func()
        self.assertEqual(output[1], "hello")
   
        

    @tier(COMMON_THREE)
    @tags("Learning")
    @number(3.0)
    def test_random(self):
        print("ehlleloe")
        result = randint(0,100)
        for i in (6, 5, 61):
            self.assertEqual(result, i, "this is my message")

    @tier(COMMON_TWO)
    @number(2.0)
    def test_random2(self):
        result = randint(0,100)
        self.assertEqual(result, 5, "this is my message")

    @tier(COMMON_ONE)
    def test_valid(self):
        self.assertEqual(5, 5, "really!")

    @tier(COMMON_ONE)
    def test_valid2(self):
        self.assertEqual(6, 6, "really!")


if __name__ == '__main__':
    unittest.main()
